#!/usr/bin/python3

import trio
import ksptrio

historie = []
klienti = []
cancel_scopes = {}

async def posli_zpravu(spojeni, zprava):
    await spojeni.send_all((zprava + '\n').encode('utf-8'))

async def posli_vsem(zprava):
    for spojeni in klienti:
        await posli_zpravu(spojeni, zprava)

async def obsluz_spojeni(spojeni):
    # Odešli historii zpráv
    klienti.append(spojeni)
    try:
        with trio.CancelScope() as cs:
            for zprava in historie:
                await posli_zpravu(spojeni, zprava)

            await posli_zpravu(spojeni, "Zadej jmeno:")
            radky = ksptrio.LineReader(spojeni)
            jmeno = await radky.readline()
            cancel_scopes[jmeno] = cs
            if not jmeno: return
            await posli_vsem(f'Uzivatel {jmeno} se pripojil')

            while True:
                text = await radky.readline()
                if not text: break

                if text.startswith('kick '):
                    cil = text.split()[1]
                    await posli_vsem(f"Uživatel {jmeno} vykopnul uživatele {cil}")
                    if cil in cancel_scopes:
                        cancel_scopes[cil].cancel()
                else:
                    zprava = f'{jmeno}: {text}'
                    await posli_vsem(zprava)
                    historie.append(zprava) # přidej do historie
    finally:
        await posli_vsem(f'Uzivatel {jmeno} se odpojil')
        klienti.remove(spojeni)
        if jmeno and jmeno in cancel_scopes:
            del cancel_scopes[jmeno]


async def main():
    await trio.serve_tcp(obsluz_spojeni, 1234)

trio.run(main)


